# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date, datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncSinglePage, AsyncSinglePage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.email_security.phishguard import report_list_params
from ....types.email_security.phishguard.report_list_response import ReportListResponse

__all__ = ["ReportsResource", "AsyncReportsResource"]


class ReportsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ReportsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        end: Union[str, datetime] | Omit = omit,
        from_date: Union[str, date] | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        to_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[ReportListResponse]:
        """Retrieves PhishGuard security alert reports for a specified date range.

        Reports
        include detected threats, dispositions, and contextual information. Use for
        security monitoring and threat analysis.

        Args:
          account_id: Identifier.

          end: End of the time range (RFC3339). Takes precedence over to_date.

          from_date: Deprecated, use `start` instead. Start date in YYYY-MM-DD format.

          start: Start of the time range (RFC3339). Takes precedence over from_date.

          to_date: Deprecated, use `end` instead. End date in YYYY-MM-DD format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/phishguard/reports", account_id=account_id),
            page=SyncSinglePage[ReportListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end": end,
                        "from_date": from_date,
                        "start": start,
                        "to_date": to_date,
                    },
                    report_list_params.ReportListParams,
                ),
            ),
            model=ReportListResponse,
        )


class AsyncReportsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncReportsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str,
        end: Union[str, datetime] | Omit = omit,
        from_date: Union[str, date] | Omit = omit,
        start: Union[str, datetime] | Omit = omit,
        to_date: Union[str, date] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[ReportListResponse, AsyncSinglePage[ReportListResponse]]:
        """Retrieves PhishGuard security alert reports for a specified date range.

        Reports
        include detected threats, dispositions, and contextual information. Use for
        security monitoring and threat analysis.

        Args:
          account_id: Identifier.

          end: End of the time range (RFC3339). Takes precedence over to_date.

          from_date: Deprecated, use `start` instead. Start date in YYYY-MM-DD format.

          start: Start of the time range (RFC3339). Takes precedence over from_date.

          to_date: Deprecated, use `end` instead. End date in YYYY-MM-DD format.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/email-security/phishguard/reports", account_id=account_id),
            page=AsyncSinglePage[ReportListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "end": end,
                        "from_date": from_date,
                        "start": start,
                        "to_date": to_date,
                    },
                    report_list_params.ReportListParams,
                ),
            ),
            model=ReportListResponse,
        )


class ReportsResourceWithRawResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.list = to_raw_response_wrapper(
            reports.list,
        )


class AsyncReportsResourceWithRawResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.list = async_to_raw_response_wrapper(
            reports.list,
        )


class ReportsResourceWithStreamingResponse:
    def __init__(self, reports: ReportsResource) -> None:
        self._reports = reports

        self.list = to_streamed_response_wrapper(
            reports.list,
        )


class AsyncReportsResourceWithStreamingResponse:
    def __init__(self, reports: AsyncReportsResource) -> None:
        self._reports = reports

        self.list = async_to_streamed_response_wrapper(
            reports.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["PermissionGrant"]


class PermissionGrant(BaseModel):
    read: Optional[bool] = None

    write: Optional[bool] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["TimeTravelGetBookmarkResponse"]


class TimeTravelGetBookmarkResponse(BaseModel):
    bookmark: Optional[str] = None
    """
    A bookmark representing a specific state of the database at a specific point in
    time.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AccountSettingUpdateParams"]


class AccountSettingUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    default_usage_model: str

    green_compute: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["DatabaseUpdateParams", "ReadReplication"]


class DatabaseUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier tag."""

    read_replication: Required[ReadReplication]
    """Configuration for D1 read replication."""


class ReadReplication(TypedDict, total=False):
    """Configuration for D1 read replication."""

    mode: Required[Literal["auto", "disabled"]]
    """The read replication mode for the database.

    Use 'auto' to create replicas and allow D1 automatically place them around the
    world, or 'disabled' to not use any database replicas (it can take a few hours
    for all replicas to be deleted).
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .doh_endpoint_param import DOHEndpointParam
from .dot_endpoint_param import DOTEndpointParam
from .ipv4_endpoint_param import IPV4EndpointParam
from .ipv6_endpoint_param import IPV6EndpointParam

__all__ = ["EndpointParam"]


class EndpointParam(TypedDict, total=False):
    """Configure the destination endpoints for this location."""

    doh: Required[DOHEndpointParam]

    dot: Required[DOTEndpointParam]

    ipv4: Required[IPV4EndpointParam]

    ipv6: Required[IPV6EndpointParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .route_ases_params import RouteAsesParams as RouteAsesParams
from .route_moas_params import RouteMoasParams as RouteMoasParams
from .route_stats_params import RouteStatsParams as RouteStatsParams
from .route_ases_response import RouteAsesResponse as RouteAsesResponse
from .route_moas_response import RouteMoasResponse as RouteMoasResponse
from .route_pfx2as_params import RoutePfx2asParams as RoutePfx2asParams
from .top_prefixes_params import TopPrefixesParams as TopPrefixesParams
from .ip_timeseries_params import IPTimeseriesParams as IPTimeseriesParams
from .route_stats_response import RouteStatsResponse as RouteStatsResponse
from .route_pfx2as_response import RoutePfx2asResponse as RoutePfx2asResponse
from .route_realtime_params import RouteRealtimeParams as RouteRealtimeParams
from .top_prefixes_response import TopPrefixesResponse as TopPrefixesResponse
from .ip_timeseries_response import IPTimeseriesResponse as IPTimeseriesResponse
from .route_realtime_response import RouteRealtimeResponse as RouteRealtimeResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["Stat", "Count"]


class Count(BaseModel):
    allowed: Optional[float] = None
    """Cloudflare Images allowed usage."""

    current: Optional[float] = None
    """Cloudflare Images current usage."""


class Stat(BaseModel):
    count: Optional[Count] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .upload_create_params import UploadCreateParams as UploadCreateParams
from .upload_create_response import UploadCreateResponse as UploadCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from .rules import (
    RulesResource,
    AsyncRulesResource,
    RulesResourceWithRawResponse,
    AsyncRulesResourceWithRawResponse,
    RulesResourceWithStreamingResponse,
    AsyncRulesResourceWithStreamingResponse,
)
from .content import (
    ContentResource,
    AsyncContentResource,
    ContentResourceWithRawResponse,
    AsyncContentResourceWithRawResponse,
    ContentResourceWithStreamingResponse,
    AsyncContentResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.snippets import snippet_list_params, snippet_update_params
from ...types.snippets.snippet_get_response import SnippetGetResponse
from ...types.snippets.snippet_list_response import SnippetListResponse
from ...types.snippets.snippet_update_response import SnippetUpdateResponse

__all__ = ["SnippetsResource", "AsyncSnippetsResource"]


class SnippetsResource(SyncAPIResource):
    @cached_property
    def content(self) -> ContentResource:
        return ContentResource(self._client)

    @cached_property
    def rules(self) -> RulesResource:
        return RulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> SnippetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SnippetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SnippetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SnippetsResourceWithStreamingResponse(self)

    def update(
        self,
        snippet_name: str,
        *,
        zone_id: str,
        metadata: snippet_update_params.Metadata,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnippetUpdateResponse:
        """
        Creates or updates a snippet belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          snippet_name: Identify the snippet.

          metadata: Provide metadata about the snippet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not snippet_name:
            raise ValueError(f"Expected a non-empty value for `snippet_name` but received {snippet_name!r}")
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._put(
            path_template("/zones/{zone_id}/snippets/{snippet_name}", zone_id=zone_id, snippet_name=snippet_name),
            body=maybe_transform({"metadata": metadata}, snippet_update_params.SnippetUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                multipart_syntax="json",
                post_parser=ResultWrapper[SnippetUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnippetUpdateResponse], ResultWrapper[SnippetUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[SnippetListResponse]:
        """
        Fetches all snippets belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          page: Specifies the current page number.

          per_page: Specifies how many results to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/snippets", zone_id=zone_id),
            page=SyncV4PagePaginationArray[SnippetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    snippet_list_params.SnippetListParams,
                ),
            ),
            model=SnippetListResponse,
        )

    def delete(
        self,
        snippet_name: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a snippet belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          snippet_name: Identify the snippet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not snippet_name:
            raise ValueError(f"Expected a non-empty value for `snippet_name` but received {snippet_name!r}")
        return self._delete(
            path_template("/zones/{zone_id}/snippets/{snippet_name}", zone_id=zone_id, snippet_name=snippet_name),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    def get(
        self,
        snippet_name: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnippetGetResponse:
        """
        Fetches a snippet belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          snippet_name: Identify the snippet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not snippet_name:
            raise ValueError(f"Expected a non-empty value for `snippet_name` but received {snippet_name!r}")
        return self._get(
            path_template("/zones/{zone_id}/snippets/{snippet_name}", zone_id=zone_id, snippet_name=snippet_name),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SnippetGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnippetGetResponse], ResultWrapper[SnippetGetResponse]),
        )


class AsyncSnippetsResource(AsyncAPIResource):
    @cached_property
    def content(self) -> AsyncContentResource:
        return AsyncContentResource(self._client)

    @cached_property
    def rules(self) -> AsyncRulesResource:
        return AsyncRulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSnippetsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSnippetsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSnippetsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSnippetsResourceWithStreamingResponse(self)

    async def update(
        self,
        snippet_name: str,
        *,
        zone_id: str,
        metadata: snippet_update_params.Metadata,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnippetUpdateResponse:
        """
        Creates or updates a snippet belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          snippet_name: Identify the snippet.

          metadata: Provide metadata about the snippet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not snippet_name:
            raise ValueError(f"Expected a non-empty value for `snippet_name` but received {snippet_name!r}")
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._put(
            path_template("/zones/{zone_id}/snippets/{snippet_name}", zone_id=zone_id, snippet_name=snippet_name),
            body=await async_maybe_transform({"metadata": metadata}, snippet_update_params.SnippetUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                multipart_syntax="json",
                post_parser=ResultWrapper[SnippetUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnippetUpdateResponse], ResultWrapper[SnippetUpdateResponse]),
        )

    def list(
        self,
        *,
        zone_id: str,
        page: int | Omit = omit,
        per_page: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[SnippetListResponse, AsyncV4PagePaginationArray[SnippetListResponse]]:
        """
        Fetches all snippets belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          page: Specifies the current page number.

          per_page: Specifies how many results to return per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/snippets", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[SnippetListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    snippet_list_params.SnippetListParams,
                ),
            ),
            model=SnippetListResponse,
        )

    async def delete(
        self,
        snippet_name: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Deletes a snippet belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          snippet_name: Identify the snippet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not snippet_name:
            raise ValueError(f"Expected a non-empty value for `snippet_name` but received {snippet_name!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/snippets/{snippet_name}", zone_id=zone_id, snippet_name=snippet_name),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[object]]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    async def get(
        self,
        snippet_name: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SnippetGetResponse:
        """
        Fetches a snippet belonging to the zone.

        Args:
          zone_id: Use this field to specify the unique ID of the zone.

          snippet_name: Identify the snippet.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not snippet_name:
            raise ValueError(f"Expected a non-empty value for `snippet_name` but received {snippet_name!r}")
        return await self._get(
            path_template("/zones/{zone_id}/snippets/{snippet_name}", zone_id=zone_id, snippet_name=snippet_name),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[SnippetGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[SnippetGetResponse], ResultWrapper[SnippetGetResponse]),
        )


class SnippetsResourceWithRawResponse:
    def __init__(self, snippets: SnippetsResource) -> None:
        self._snippets = snippets

        self.update = to_raw_response_wrapper(
            snippets.update,
        )
        self.list = to_raw_response_wrapper(
            snippets.list,
        )
        self.delete = to_raw_response_wrapper(
            snippets.delete,
        )
        self.get = to_raw_response_wrapper(
            snippets.get,
        )

    @cached_property
    def content(self) -> ContentResourceWithRawResponse:
        return ContentResourceWithRawResponse(self._snippets.content)

    @cached_property
    def rules(self) -> RulesResourceWithRawResponse:
        return RulesResourceWithRawResponse(self._snippets.rules)


class AsyncSnippetsResourceWithRawResponse:
    def __init__(self, snippets: AsyncSnippetsResource) -> None:
        self._snippets = snippets

        self.update = async_to_raw_response_wrapper(
            snippets.update,
        )
        self.list = async_to_raw_response_wrapper(
            snippets.list,
        )
        self.delete = async_to_raw_response_wrapper(
            snippets.delete,
        )
        self.get = async_to_raw_response_wrapper(
            snippets.get,
        )

    @cached_property
    def content(self) -> AsyncContentResourceWithRawResponse:
        return AsyncContentResourceWithRawResponse(self._snippets.content)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithRawResponse:
        return AsyncRulesResourceWithRawResponse(self._snippets.rules)


class SnippetsResourceWithStreamingResponse:
    def __init__(self, snippets: SnippetsResource) -> None:
        self._snippets = snippets

        self.update = to_streamed_response_wrapper(
            snippets.update,
        )
        self.list = to_streamed_response_wrapper(
            snippets.list,
        )
        self.delete = to_streamed_response_wrapper(
            snippets.delete,
        )
        self.get = to_streamed_response_wrapper(
            snippets.get,
        )

    @cached_property
    def content(self) -> ContentResourceWithStreamingResponse:
        return ContentResourceWithStreamingResponse(self._snippets.content)

    @cached_property
    def rules(self) -> RulesResourceWithStreamingResponse:
        return RulesResourceWithStreamingResponse(self._snippets.rules)


class AsyncSnippetsResourceWithStreamingResponse:
    def __init__(self, snippets: AsyncSnippetsResource) -> None:
        self._snippets = snippets

        self.update = async_to_streamed_response_wrapper(
            snippets.update,
        )
        self.list = async_to_streamed_response_wrapper(
            snippets.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            snippets.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            snippets.get,
        )

    @cached_property
    def content(self) -> AsyncContentResourceWithStreamingResponse:
        return AsyncContentResourceWithStreamingResponse(self._snippets.content)

    @cached_property
    def rules(self) -> AsyncRulesResourceWithStreamingResponse:
        return AsyncRulesResourceWithStreamingResponse(self._snippets.rules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Iterable
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ScanBulkCreateParams", "Body"]


class ScanBulkCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    body: Iterable[Body]
    """List of urls to scan (up to a 100)."""


class Body(TypedDict, total=False):
    url: Required[str]

    agent_readiness: Annotated[bool, PropertyInfo(alias="agentReadiness")]
    """Enable agent readiness checks."""

    customagent: str

    custom_headers: Annotated[Dict[str, str], PropertyInfo(alias="customHeaders")]
    """Set custom headers."""

    referer: str

    screenshots_resolutions: Annotated[
        List[Literal["desktop", "mobile", "tablet"]], PropertyInfo(alias="screenshotsResolutions")
    ]
    """Take multiple screenshots targeting different device types."""

    visibility: Literal["Public", "Unlisted"]
    """
    The option `Public` means it will be included in listings like recent scans and
    search results. `Unlisted` means it will not be included in the aforementioned
    listings, users will need to have the scan's ID to access it. A a scan will be
    automatically marked as unlisted if it fails, if it contains potential PII or
    other sensitive material.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DeviceMatch"]


class DeviceMatch(BaseModel):
    platform: Optional[Literal["windows", "mac", "linux", "android", "ios", "chromeos"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .bytime_get_params import BytimeGetParams as BytimeGetParams
from .summary_get_params import SummaryGetParams as SummaryGetParams
from .bytime_get_response import BytimeGetResponse as BytimeGetResponse
from .summary_get_response import SummaryGetResponse as SummaryGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["LocationListParams"]


class LocationListParams(TypedDict, total=False):
    continent: Literal["AF", "AS", "EU", "NA", "OC", "SA"]
    """Filters results by continent code."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    limit: int
    """Limits the number of objects returned in the response."""

    location: str
    """Filters results by location.

    Specify a comma-separated list of alpha-2 location codes.
    """

    offset: int
    """Skips the specified number of objects before fetching the results."""

    region: str
    """Filters results by region."""

    subregion: str
    """Filters results by subregion."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["PackageListParams"]


class PackageListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    direction: Literal["asc", "desc"]
    """The direction used to sort returned packages."""

    match: Literal["any", "all"]
    """When set to `all`, all the search requirements must match.

    When set to `any`, only one of the search requirements has to match.
    """

    name: str
    """The name of the WAF package."""

    order: Literal["name"]
    """The field used to sort returned packages."""

    page: float
    """The page number of paginated results."""

    per_page: float
    """The number of packages per page."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["SkipConfiguration"]


class SkipConfiguration(BaseModel):
    """Content types to exclude from context analysis and return all matches."""

    files: bool
    """If the content type is a file, skip context analysis and return all matches."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Type, Optional, cast
from typing_extensions import Literal, overload

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.spectrum import app_list_params, app_create_params, app_update_params
from ...types.spectrum.dns_param import DNSParam
from ...types.spectrum.edge_ips_param import EdgeIPsParam
from ...types.spectrum.app_get_response import AppGetResponse
from ...types.spectrum.origin_dns_param import OriginDNSParam
from ...types.spectrum.app_list_response import AppListResponse
from ...types.spectrum.origin_port_param import OriginPortParam
from ...types.spectrum.app_create_response import AppCreateResponse
from ...types.spectrum.app_delete_response import AppDeleteResponse
from ...types.spectrum.app_update_response import AppUpdateResponse

__all__ = ["AppsResource", "AsyncAppsResource"]


class AppsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AppsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AppsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AppsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AppsResourceWithStreamingResponse(self)

    @overload
    def create(
        self,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"],
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        """
        Creates a new Spectrum application from a configuration using a name for the
        origin.

        Args:
          zone_id: Zone identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          traffic_type: Determines how data travels from the edge to your origin. When set to "direct",
              Spectrum will send traffic directly to your origin, and the application's type
              is derived from the `protocol`. When set to "http" or "https", Spectrum will
              apply Cloudflare's HTTP/HTTPS features as it sends traffic to your origin, and
              the application type matches this property exactly.

          argo_smart_routing: Enables Argo Smart Routing for this application. Notes: Only available for TCP
              applications with traffic_type set to "direct".

          edge_ips: The anycast edge IP configuration for the hostname of this application.

          ip_firewall: Enables IP Access Rules for this application. Notes: Only available for TCP
              applications.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          origin_dns: The name and type of DNS record for the Spectrum application.

          origin_port: The destination port at the origin. Only specified in conjunction with
              origin_dns. May use an integer to specify a single origin port, for example
              `1000`, or a string to specify a range of origin ports, for example
              `"1000-2000"`. Notes: If specifying a port range, the number of ports in the
              range must match the number of ports specified in the "protocol" field.

          proxy_protocol: Enables Proxy Protocol to the origin. Refer to
              [Enable Proxy protocol](https://developers.cloudflare.com/spectrum/getting-started/proxy-protocol/)
              for implementation details on PROXY Protocol V1, PROXY Protocol V2, and Simple
              Proxy Protocol.

          tls: The type of TLS termination associated with the application.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create(
        self,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        """
        Creates a new Spectrum application from a configuration using a name for the
        origin.

        Args:
          zone_id: Zone identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id", "dns", "protocol", "traffic_type"], ["zone_id", "dns", "protocol"])
    def create(
        self,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"] | Omit = omit,
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[AppCreateResponse],
            self._post(
                path_template("/zones/{zone_id}/spectrum/apps", zone_id=zone_id),
                body=maybe_transform(
                    {
                        "dns": dns,
                        "protocol": protocol,
                        "traffic_type": traffic_type,
                        "argo_smart_routing": argo_smart_routing,
                        "edge_ips": edge_ips,
                        "ip_firewall": ip_firewall,
                        "origin_direct": origin_direct,
                        "origin_dns": origin_dns,
                        "origin_port": origin_port,
                        "proxy_protocol": proxy_protocol,
                        "tls": tls,
                    },
                    app_create_params.AppCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[AppCreateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[AppCreateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    def update(
        self,
        app_id: str,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"],
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        """
        Updates a previously existing application's configuration that uses a name for
        the origin.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          traffic_type: Determines how data travels from the edge to your origin. When set to "direct",
              Spectrum will send traffic directly to your origin, and the application's type
              is derived from the `protocol`. When set to "http" or "https", Spectrum will
              apply Cloudflare's HTTP/HTTPS features as it sends traffic to your origin, and
              the application type matches this property exactly.

          argo_smart_routing: Enables Argo Smart Routing for this application. Notes: Only available for TCP
              applications with traffic_type set to "direct".

          edge_ips: The anycast edge IP configuration for the hostname of this application.

          ip_firewall: Enables IP Access Rules for this application. Notes: Only available for TCP
              applications.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          origin_dns: The name and type of DNS record for the Spectrum application.

          origin_port: The destination port at the origin. Only specified in conjunction with
              origin_dns. May use an integer to specify a single origin port, for example
              `1000`, or a string to specify a range of origin ports, for example
              `"1000-2000"`. Notes: If specifying a port range, the number of ports in the
              range must match the number of ports specified in the "protocol" field.

          proxy_protocol: Enables Proxy Protocol to the origin. Refer to
              [Enable Proxy protocol](https://developers.cloudflare.com/spectrum/getting-started/proxy-protocol/)
              for implementation details on PROXY Protocol V1, PROXY Protocol V2, and Simple
              Proxy Protocol.

          tls: The type of TLS termination associated with the application.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def update(
        self,
        app_id: str,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        """
        Updates a previously existing application's configuration that uses a name for
        the origin.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id", "dns", "protocol", "traffic_type"], ["zone_id", "dns", "protocol"])
    def update(
        self,
        app_id: str,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"] | Omit = omit,
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return cast(
            Optional[AppUpdateResponse],
            self._put(
                path_template("/zones/{zone_id}/spectrum/apps/{app_id}", zone_id=zone_id, app_id=app_id),
                body=maybe_transform(
                    {
                        "dns": dns,
                        "protocol": protocol,
                        "traffic_type": traffic_type,
                        "argo_smart_routing": argo_smart_routing,
                        "edge_ips": edge_ips,
                        "ip_firewall": ip_firewall,
                        "origin_direct": origin_direct,
                        "origin_dns": origin_dns,
                        "origin_port": origin_port,
                        "proxy_protocol": proxy_protocol,
                        "tls": tls,
                    },
                    app_update_params.AppUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[AppUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[AppUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["protocol", "app_id", "created_on", "modified_on", "dns"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[AppListResponse]:
        """
        Retrieves a list of currently existing Spectrum applications inside a zone.

        Args:
          zone_id: Zone identifier.

          direction: Sets the direction by which results are ordered.

          order: Application field by which results are ordered.

          page: Page number of paginated results. This parameter is required in order to use
              other pagination parameters. If included in the query, `result_info` will be
              present in the response.

          per_page: Sets the maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/spectrum/apps", zone_id=zone_id),
            page=SyncV4PagePaginationArray[AppListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    app_list_params.AppListParams,
                ),
            ),
            model=cast(Any, AppListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    def delete(
        self,
        app_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppDeleteResponse]:
        """
        Deletes a previously existing application.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._delete(
            path_template("/zones/{zone_id}/spectrum/apps/{app_id}", zone_id=zone_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppDeleteResponse]], ResultWrapper[AppDeleteResponse]),
        )

    def get(
        self,
        app_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppGetResponse]:
        """
        Gets the application configuration of a specific application inside a zone.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return cast(
            Optional[AppGetResponse],
            self._get(
                path_template("/zones/{zone_id}/spectrum/apps/{app_id}", zone_id=zone_id, app_id=app_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[AppGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[AppGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AsyncAppsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAppsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAppsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAppsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAppsResourceWithStreamingResponse(self)

    @overload
    async def create(
        self,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"],
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        """
        Creates a new Spectrum application from a configuration using a name for the
        origin.

        Args:
          zone_id: Zone identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          traffic_type: Determines how data travels from the edge to your origin. When set to "direct",
              Spectrum will send traffic directly to your origin, and the application's type
              is derived from the `protocol`. When set to "http" or "https", Spectrum will
              apply Cloudflare's HTTP/HTTPS features as it sends traffic to your origin, and
              the application type matches this property exactly.

          argo_smart_routing: Enables Argo Smart Routing for this application. Notes: Only available for TCP
              applications with traffic_type set to "direct".

          edge_ips: The anycast edge IP configuration for the hostname of this application.

          ip_firewall: Enables IP Access Rules for this application. Notes: Only available for TCP
              applications.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          origin_dns: The name and type of DNS record for the Spectrum application.

          origin_port: The destination port at the origin. Only specified in conjunction with
              origin_dns. May use an integer to specify a single origin port, for example
              `1000`, or a string to specify a range of origin ports, for example
              `"1000-2000"`. Notes: If specifying a port range, the number of ports in the
              range must match the number of ports specified in the "protocol" field.

          proxy_protocol: Enables Proxy Protocol to the origin. Refer to
              [Enable Proxy protocol](https://developers.cloudflare.com/spectrum/getting-started/proxy-protocol/)
              for implementation details on PROXY Protocol V1, PROXY Protocol V2, and Simple
              Proxy Protocol.

          tls: The type of TLS termination associated with the application.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create(
        self,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        """
        Creates a new Spectrum application from a configuration using a name for the
        origin.

        Args:
          zone_id: Zone identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id", "dns", "protocol", "traffic_type"], ["zone_id", "dns", "protocol"])
    async def create(
        self,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"] | Omit = omit,
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppCreateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return cast(
            Optional[AppCreateResponse],
            await self._post(
                path_template("/zones/{zone_id}/spectrum/apps", zone_id=zone_id),
                body=await async_maybe_transform(
                    {
                        "dns": dns,
                        "protocol": protocol,
                        "traffic_type": traffic_type,
                        "argo_smart_routing": argo_smart_routing,
                        "edge_ips": edge_ips,
                        "ip_firewall": ip_firewall,
                        "origin_direct": origin_direct,
                        "origin_dns": origin_dns,
                        "origin_port": origin_port,
                        "proxy_protocol": proxy_protocol,
                        "tls": tls,
                    },
                    app_create_params.AppCreateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[AppCreateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[AppCreateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    @overload
    async def update(
        self,
        app_id: str,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"],
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        """
        Updates a previously existing application's configuration that uses a name for
        the origin.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          traffic_type: Determines how data travels from the edge to your origin. When set to "direct",
              Spectrum will send traffic directly to your origin, and the application's type
              is derived from the `protocol`. When set to "http" or "https", Spectrum will
              apply Cloudflare's HTTP/HTTPS features as it sends traffic to your origin, and
              the application type matches this property exactly.

          argo_smart_routing: Enables Argo Smart Routing for this application. Notes: Only available for TCP
              applications with traffic_type set to "direct".

          edge_ips: The anycast edge IP configuration for the hostname of this application.

          ip_firewall: Enables IP Access Rules for this application. Notes: Only available for TCP
              applications.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          origin_dns: The name and type of DNS record for the Spectrum application.

          origin_port: The destination port at the origin. Only specified in conjunction with
              origin_dns. May use an integer to specify a single origin port, for example
              `1000`, or a string to specify a range of origin ports, for example
              `"1000-2000"`. Notes: If specifying a port range, the number of ports in the
              range must match the number of ports specified in the "protocol" field.

          proxy_protocol: Enables Proxy Protocol to the origin. Refer to
              [Enable Proxy protocol](https://developers.cloudflare.com/spectrum/getting-started/proxy-protocol/)
              for implementation details on PROXY Protocol V1, PROXY Protocol V2, and Simple
              Proxy Protocol.

          tls: The type of TLS termination associated with the application.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def update(
        self,
        app_id: str,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        """
        Updates a previously existing application's configuration that uses a name for
        the origin.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          dns: The name and type of DNS record for the Spectrum application.

          protocol: The port configuration at Cloudflare's edge. May specify a single port, for
              example `"tcp/1000"`, or a range of ports, for example `"tcp/1000-2000"`.

          origin_direct: List of origin IP addresses. Array may contain multiple IP addresses for load
              balancing.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["zone_id", "dns", "protocol", "traffic_type"], ["zone_id", "dns", "protocol"])
    async def update(
        self,
        app_id: str,
        *,
        zone_id: str,
        dns: DNSParam,
        protocol: str,
        traffic_type: Literal["direct", "http", "https"] | Omit = omit,
        argo_smart_routing: bool | Omit = omit,
        edge_ips: EdgeIPsParam | Omit = omit,
        ip_firewall: bool | Omit = omit,
        origin_direct: SequenceNotStr[str] | Omit = omit,
        origin_dns: OriginDNSParam | Omit = omit,
        origin_port: OriginPortParam | Omit = omit,
        proxy_protocol: Literal["off", "v1", "v2", "simple"] | Omit = omit,
        tls: Literal["off", "flexible", "full", "strict"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppUpdateResponse]:
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return cast(
            Optional[AppUpdateResponse],
            await self._put(
                path_template("/zones/{zone_id}/spectrum/apps/{app_id}", zone_id=zone_id, app_id=app_id),
                body=await async_maybe_transform(
                    {
                        "dns": dns,
                        "protocol": protocol,
                        "traffic_type": traffic_type,
                        "argo_smart_routing": argo_smart_routing,
                        "edge_ips": edge_ips,
                        "ip_firewall": ip_firewall,
                        "origin_direct": origin_direct,
                        "origin_dns": origin_dns,
                        "origin_port": origin_port,
                        "proxy_protocol": proxy_protocol,
                        "tls": tls,
                    },
                    app_update_params.AppUpdateParams,
                ),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[AppUpdateResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[AppUpdateResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )

    def list(
        self,
        *,
        zone_id: str,
        direction: Literal["asc", "desc"] | Omit = omit,
        order: Literal["protocol", "app_id", "created_on", "modified_on", "dns"] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AppListResponse, AsyncV4PagePaginationArray[AppListResponse]]:
        """
        Retrieves a list of currently existing Spectrum applications inside a zone.

        Args:
          zone_id: Zone identifier.

          direction: Sets the direction by which results are ordered.

          order: Application field by which results are ordered.

          page: Page number of paginated results. This parameter is required in order to use
              other pagination parameters. If included in the query, `result_info` will be
              present in the response.

          per_page: Sets the maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get_api_list(
            path_template("/zones/{zone_id}/spectrum/apps", zone_id=zone_id),
            page=AsyncV4PagePaginationArray[AppListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "direction": direction,
                        "order": order,
                        "page": page,
                        "per_page": per_page,
                    },
                    app_list_params.AppListParams,
                ),
            ),
            model=cast(Any, AppListResponse),  # Union types cannot be passed in as arguments in the type system
        )

    async def delete(
        self,
        app_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppDeleteResponse]:
        """
        Deletes a previously existing application.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._delete(
            path_template("/zones/{zone_id}/spectrum/apps/{app_id}", zone_id=zone_id, app_id=app_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[AppDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[AppDeleteResponse]], ResultWrapper[AppDeleteResponse]),
        )

    async def get(
        self,
        app_id: str,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AppGetResponse]:
        """
        Gets the application configuration of a specific application inside a zone.

        Args:
          zone_id: Zone identifier.

          app_id: App identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return cast(
            Optional[AppGetResponse],
            await self._get(
                path_template("/zones/{zone_id}/spectrum/apps/{app_id}", zone_id=zone_id, app_id=app_id),
                options=make_request_options(
                    extra_headers=extra_headers,
                    extra_query=extra_query,
                    extra_body=extra_body,
                    timeout=timeout,
                    post_parser=ResultWrapper[Optional[AppGetResponse]]._unwrapper,
                ),
                cast_to=cast(
                    Any, ResultWrapper[AppGetResponse]
                ),  # Union types cannot be passed in as arguments in the type system
            ),
        )


class AppsResourceWithRawResponse:
    def __init__(self, apps: AppsResource) -> None:
        self._apps = apps

        self.create = to_raw_response_wrapper(
            apps.create,
        )
        self.update = to_raw_response_wrapper(
            apps.update,
        )
        self.list = to_raw_response_wrapper(
            apps.list,
        )
        self.delete = to_raw_response_wrapper(
            apps.delete,
        )
        self.get = to_raw_response_wrapper(
            apps.get,
        )


class AsyncAppsResourceWithRawResponse:
    def __init__(self, apps: AsyncAppsResource) -> None:
        self._apps = apps

        self.create = async_to_raw_response_wrapper(
            apps.create,
        )
        self.update = async_to_raw_response_wrapper(
            apps.update,
        )
        self.list = async_to_raw_response_wrapper(
            apps.list,
        )
        self.delete = async_to_raw_response_wrapper(
            apps.delete,
        )
        self.get = async_to_raw_response_wrapper(
            apps.get,
        )


class AppsResourceWithStreamingResponse:
    def __init__(self, apps: AppsResource) -> None:
        self._apps = apps

        self.create = to_streamed_response_wrapper(
            apps.create,
        )
        self.update = to_streamed_response_wrapper(
            apps.update,
        )
        self.list = to_streamed_response_wrapper(
            apps.list,
        )
        self.delete = to_streamed_response_wrapper(
            apps.delete,
        )
        self.get = to_streamed_response_wrapper(
            apps.get,
        )


class AsyncAppsResourceWithStreamingResponse:
    def __init__(self, apps: AsyncAppsResource) -> None:
        self._apps = apps

        self.create = async_to_streamed_response_wrapper(
            apps.create,
        )
        self.update = async_to_streamed_response_wrapper(
            apps.update,
        )
        self.list = async_to_streamed_response_wrapper(
            apps.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            apps.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            apps.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["MessagePullParams"]


class MessagePullParams(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    batch_size: float
    """The maximum number of messages to include in a batch."""

    visibility_timeout_ms: float
    """The number of milliseconds that a message is exclusively leased.

    After the timeout, the message becomes available for another attempt.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["TokenListParams"]


class TokenListParams(TypedDict, total=False):
    account_id: Required[str]

    page: int
    """Page number (1-indexed)."""

    per_page: int
    """Number of results per page."""

    search: str
    """Filter tokens whose name contains this string (case-insensitive)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .schemas import (
    SchemasResource,
    AsyncSchemasResource,
    SchemasResourceWithRawResponse,
    AsyncSchemasResourceWithRawResponse,
    SchemasResourceWithStreamingResponse,
    AsyncSchemasResourceWithStreamingResponse,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from .labels.labels import (
    LabelsResource,
    AsyncLabelsResource,
    LabelsResourceWithRawResponse,
    AsyncLabelsResourceWithRawResponse,
    LabelsResourceWithStreamingResponse,
    AsyncLabelsResourceWithStreamingResponse,
)
from .configurations import (
    ConfigurationsResource,
    AsyncConfigurationsResource,
    ConfigurationsResourceWithRawResponse,
    AsyncConfigurationsResourceWithRawResponse,
    ConfigurationsResourceWithStreamingResponse,
    AsyncConfigurationsResourceWithStreamingResponse,
)
from .settings.settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .discovery.discovery import (
    DiscoveryResource,
    AsyncDiscoveryResource,
    DiscoveryResourceWithRawResponse,
    AsyncDiscoveryResourceWithRawResponse,
    DiscoveryResourceWithStreamingResponse,
    AsyncDiscoveryResourceWithStreamingResponse,
)
from .operations.operations import (
    OperationsResource,
    AsyncOperationsResource,
    OperationsResourceWithRawResponse,
    AsyncOperationsResourceWithRawResponse,
    OperationsResourceWithStreamingResponse,
    AsyncOperationsResourceWithStreamingResponse,
)
from .user_schemas.user_schemas import (
    UserSchemasResource,
    AsyncUserSchemasResource,
    UserSchemasResourceWithRawResponse,
    AsyncUserSchemasResourceWithRawResponse,
    UserSchemasResourceWithStreamingResponse,
    AsyncUserSchemasResourceWithStreamingResponse,
)
from .expression_template.expression_template import (
    ExpressionTemplateResource,
    AsyncExpressionTemplateResource,
    ExpressionTemplateResourceWithRawResponse,
    AsyncExpressionTemplateResourceWithRawResponse,
    ExpressionTemplateResourceWithStreamingResponse,
    AsyncExpressionTemplateResourceWithStreamingResponse,
)

__all__ = ["APIGatewayResource", "AsyncAPIGatewayResource"]


class APIGatewayResource(SyncAPIResource):
    @cached_property
    def configurations(self) -> ConfigurationsResource:
        return ConfigurationsResource(self._client)

    @cached_property
    def discovery(self) -> DiscoveryResource:
        return DiscoveryResource(self._client)

    @cached_property
    def labels(self) -> LabelsResource:
        return LabelsResource(self._client)

    @cached_property
    def operations(self) -> OperationsResource:
        return OperationsResource(self._client)

    @cached_property
    def schemas(self) -> SchemasResource:
        return SchemasResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def user_schemas(self) -> UserSchemasResource:
        return UserSchemasResource(self._client)

    @cached_property
    def expression_template(self) -> ExpressionTemplateResource:
        return ExpressionTemplateResource(self._client)

    @cached_property
    def with_raw_response(self) -> APIGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return APIGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> APIGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return APIGatewayResourceWithStreamingResponse(self)


class AsyncAPIGatewayResource(AsyncAPIResource):
    @cached_property
    def configurations(self) -> AsyncConfigurationsResource:
        return AsyncConfigurationsResource(self._client)

    @cached_property
    def discovery(self) -> AsyncDiscoveryResource:
        return AsyncDiscoveryResource(self._client)

    @cached_property
    def labels(self) -> AsyncLabelsResource:
        return AsyncLabelsResource(self._client)

    @cached_property
    def operations(self) -> AsyncOperationsResource:
        return AsyncOperationsResource(self._client)

    @cached_property
    def schemas(self) -> AsyncSchemasResource:
        return AsyncSchemasResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def user_schemas(self) -> AsyncUserSchemasResource:
        return AsyncUserSchemasResource(self._client)

    @cached_property
    def expression_template(self) -> AsyncExpressionTemplateResource:
        return AsyncExpressionTemplateResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAPIGatewayResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAPIGatewayResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAPIGatewayResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAPIGatewayResourceWithStreamingResponse(self)


class APIGatewayResourceWithRawResponse:
    def __init__(self, api_gateway: APIGatewayResource) -> None:
        self._api_gateway = api_gateway

    @cached_property
    def configurations(self) -> ConfigurationsResourceWithRawResponse:
        return ConfigurationsResourceWithRawResponse(self._api_gateway.configurations)

    @cached_property
    def discovery(self) -> DiscoveryResourceWithRawResponse:
        return DiscoveryResourceWithRawResponse(self._api_gateway.discovery)

    @cached_property
    def labels(self) -> LabelsResourceWithRawResponse:
        return LabelsResourceWithRawResponse(self._api_gateway.labels)

    @cached_property
    def operations(self) -> OperationsResourceWithRawResponse:
        return OperationsResourceWithRawResponse(self._api_gateway.operations)

    @cached_property
    def schemas(self) -> SchemasResourceWithRawResponse:
        return SchemasResourceWithRawResponse(self._api_gateway.schemas)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._api_gateway.settings)

    @cached_property
    def user_schemas(self) -> UserSchemasResourceWithRawResponse:
        return UserSchemasResourceWithRawResponse(self._api_gateway.user_schemas)

    @cached_property
    def expression_template(self) -> ExpressionTemplateResourceWithRawResponse:
        return ExpressionTemplateResourceWithRawResponse(self._api_gateway.expression_template)


class AsyncAPIGatewayResourceWithRawResponse:
    def __init__(self, api_gateway: AsyncAPIGatewayResource) -> None:
        self._api_gateway = api_gateway

    @cached_property
    def configurations(self) -> AsyncConfigurationsResourceWithRawResponse:
        return AsyncConfigurationsResourceWithRawResponse(self._api_gateway.configurations)

    @cached_property
    def discovery(self) -> AsyncDiscoveryResourceWithRawResponse:
        return AsyncDiscoveryResourceWithRawResponse(self._api_gateway.discovery)

    @cached_property
    def labels(self) -> AsyncLabelsResourceWithRawResponse:
        return AsyncLabelsResourceWithRawResponse(self._api_gateway.labels)

    @cached_property
    def operations(self) -> AsyncOperationsResourceWithRawResponse:
        return AsyncOperationsResourceWithRawResponse(self._api_gateway.operations)

    @cached_property
    def schemas(self) -> AsyncSchemasResourceWithRawResponse:
        return AsyncSchemasResourceWithRawResponse(self._api_gateway.schemas)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._api_gateway.settings)

    @cached_property
    def user_schemas(self) -> AsyncUserSchemasResourceWithRawResponse:
        return AsyncUserSchemasResourceWithRawResponse(self._api_gateway.user_schemas)

    @cached_property
    def expression_template(self) -> AsyncExpressionTemplateResourceWithRawResponse:
        return AsyncExpressionTemplateResourceWithRawResponse(self._api_gateway.expression_template)


class APIGatewayResourceWithStreamingResponse:
    def __init__(self, api_gateway: APIGatewayResource) -> None:
        self._api_gateway = api_gateway

    @cached_property
    def configurations(self) -> ConfigurationsResourceWithStreamingResponse:
        return ConfigurationsResourceWithStreamingResponse(self._api_gateway.configurations)

    @cached_property
    def discovery(self) -> DiscoveryResourceWithStreamingResponse:
        return DiscoveryResourceWithStreamingResponse(self._api_gateway.discovery)

    @cached_property
    def labels(self) -> LabelsResourceWithStreamingResponse:
        return LabelsResourceWithStreamingResponse(self._api_gateway.labels)

    @cached_property
    def operations(self) -> OperationsResourceWithStreamingResponse:
        return OperationsResourceWithStreamingResponse(self._api_gateway.operations)

    @cached_property
    def schemas(self) -> SchemasResourceWithStreamingResponse:
        return SchemasResourceWithStreamingResponse(self._api_gateway.schemas)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._api_gateway.settings)

    @cached_property
    def user_schemas(self) -> UserSchemasResourceWithStreamingResponse:
        return UserSchemasResourceWithStreamingResponse(self._api_gateway.user_schemas)

    @cached_property
    def expression_template(self) -> ExpressionTemplateResourceWithStreamingResponse:
        return ExpressionTemplateResourceWithStreamingResponse(self._api_gateway.expression_template)


class AsyncAPIGatewayResourceWithStreamingResponse:
    def __init__(self, api_gateway: AsyncAPIGatewayResource) -> None:
        self._api_gateway = api_gateway

    @cached_property
    def configurations(self) -> AsyncConfigurationsResourceWithStreamingResponse:
        return AsyncConfigurationsResourceWithStreamingResponse(self._api_gateway.configurations)

    @cached_property
    def discovery(self) -> AsyncDiscoveryResourceWithStreamingResponse:
        return AsyncDiscoveryResourceWithStreamingResponse(self._api_gateway.discovery)

    @cached_property
    def labels(self) -> AsyncLabelsResourceWithStreamingResponse:
        return AsyncLabelsResourceWithStreamingResponse(self._api_gateway.labels)

    @cached_property
    def operations(self) -> AsyncOperationsResourceWithStreamingResponse:
        return AsyncOperationsResourceWithStreamingResponse(self._api_gateway.operations)

    @cached_property
    def schemas(self) -> AsyncSchemasResourceWithStreamingResponse:
        return AsyncSchemasResourceWithStreamingResponse(self._api_gateway.schemas)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._api_gateway.settings)

    @cached_property
    def user_schemas(self) -> AsyncUserSchemasResourceWithStreamingResponse:
        return AsyncUserSchemasResourceWithStreamingResponse(self._api_gateway.user_schemas)

    @cached_property
    def expression_template(self) -> AsyncExpressionTemplateResourceWithStreamingResponse:
        return AsyncExpressionTemplateResourceWithStreamingResponse(self._api_gateway.expression_template)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .rayid_get_params import RayIDGetParams as RayIDGetParams
from .rayid_get_response import RayIDGetResponse as RayIDGetResponse
from .received_get_params import ReceivedGetParams as ReceivedGetParams
from .received_get_response import ReceivedGetResponse as ReceivedGetResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .web_crawler_summary_params import WebCrawlerSummaryParams as WebCrawlerSummaryParams
from .web_crawler_summary_response import WebCrawlerSummaryResponse as WebCrawlerSummaryResponse
from .web_crawler_timeseries_groups_params import WebCrawlerTimeseriesGroupsParams as WebCrawlerTimeseriesGroupsParams
from .web_crawler_timeseries_groups_response import (
    WebCrawlerTimeseriesGroupsResponse as WebCrawlerTimeseriesGroupsResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GroupRuleParam", "Group"]


class Group(TypedDict, total=False):
    id: Required[str]
    """The ID of a previously created Access group."""


class GroupRuleParam(TypedDict, total=False):
    """Matches an Access group."""

    group: Required[Group]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["GatewayListResponse"]


class GatewayListResponse(BaseModel):
    id: Optional[str] = None
    """Specify the Cloudflare account ID."""

    gateway_tag: Optional[str] = None
    """Specify the gateway internal ID."""

    provider_name: Optional[str] = None
    """Specify the provider name (usually Cloudflare)."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BotListResponse", "Bot"]


class Bot(BaseModel):
    category: str
    """The category of the bot."""

    description: str
    """A summary for the bot (e.g., purpose)."""

    kind: str
    """The kind of the bot."""

    name: str
    """The name of the bot."""

    operator: str
    """The organization that owns and operates the bot."""

    slug: str
    """A kebab-case identifier derived from the bot name."""

    user_agent_patterns: List[str] = FieldInfo(alias="userAgentPatterns")


class BotListResponse(BaseModel):
    bots: List[Bot]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["LiveInput", "Recording", "Rtmps", "RtmpsPlayback", "Srt", "SrtPlayback", "WebRtc", "WebRtcPlayback"]


class Recording(BaseModel):
    """Records the input to a Cloudflare Stream video.

    Behavior depends on the mode. In most cases, the video will initially be viewable as a live video and transition to on-demand after a condition is satisfied.
    """

    allowed_origins: Optional[List[str]] = FieldInfo(alias="allowedOrigins", default=None)
    """Lists the origins allowed to display videos created with this input.

    Enter allowed origin domains in an array and use `*` for wildcard subdomains. An
    empty array allows videos to be viewed on any origin.
    """

    hide_live_viewer_count: Optional[bool] = FieldInfo(alias="hideLiveViewerCount", default=None)
    """
    Disables reporting the number of live viewers when this property is set to
    `true`.
    """

    mode: Optional[Literal["off", "automatic"]] = None
    """Specifies the recording behavior for the live input.

    Set this value to `off` to prevent a recording. Set the value to `automatic` to
    begin a recording and transition to on-demand after Stream Live stops receiving
    input.
    """

    require_signed_urls: Optional[bool] = FieldInfo(alias="requireSignedURLs", default=None)
    """
    Indicates if a video using the live input has the `requireSignedURLs` property
    set. Also enforces access controls on any video recording of the livestream with
    the live input.
    """

    timeout_seconds: Optional[int] = FieldInfo(alias="timeoutSeconds", default=None)
    """
    Determines the amount of time a live input configured in `automatic` mode should
    wait before a recording transitions from live to on-demand. `0` is recommended
    for most use cases and indicates the platform default should be used.
    """


class Rtmps(BaseModel):
    """Details for streaming to an live input using RTMPS."""

    stream_key: Optional[str] = FieldInfo(alias="streamKey", default=None)
    """The secret key to use when streaming via RTMPS to a live input."""

    url: Optional[str] = None
    """The RTMPS URL you provide to the broadcaster, which they stream live video to."""


class RtmpsPlayback(BaseModel):
    """Details for playback from an live input using RTMPS."""

    stream_key: Optional[str] = FieldInfo(alias="streamKey", default=None)
    """The secret key to use for playback via RTMPS."""

    url: Optional[str] = None
    """The URL used to play live video over RTMPS."""


class Srt(BaseModel):
    """Details for streaming to a live input using SRT."""

    passphrase: Optional[str] = None
    """The secret key to use when streaming via SRT to a live input."""

    stream_id: Optional[str] = FieldInfo(alias="streamId", default=None)
    """The identifier of the live input to use when streaming via SRT."""

    url: Optional[str] = None
    """The SRT URL you provide to the broadcaster, which they stream live video to."""


class SrtPlayback(BaseModel):
    """Details for playback from an live input using SRT."""

    passphrase: Optional[str] = None
    """The secret key to use for playback via SRT."""

    stream_id: Optional[str] = FieldInfo(alias="streamId", default=None)
    """The identifier of the live input to use for playback via SRT."""

    url: Optional[str] = None
    """The URL used to play live video over SRT."""


class WebRtc(BaseModel):
    """Details for streaming to a live input using WebRTC."""

    url: Optional[str] = None
    """The WebRTC URL you provide to the broadcaster, which they stream live video to."""


class WebRtcPlayback(BaseModel):
    """Details for playback from a live input using WebRTC."""

    url: Optional[str] = None
    """The URL used to play live video over WebRTC."""


class LiveInput(BaseModel):
    """Details about a live input."""

    created: Optional[datetime] = None
    """The date and time the live input was created."""

    delete_recording_after_days: Optional[float] = FieldInfo(alias="deleteRecordingAfterDays", default=None)
    """
    Indicates the number of days after which the live inputs recordings will be
    deleted. When a stream completes and the recording is ready, the value is used
    to calculate a scheduled deletion date for that recording. Omit the field to
    indicate no change, or include with a `null` value to remove an existing
    scheduled deletion.
    """

    enabled: Optional[bool] = None
    """Indicates whether the live input is enabled and can accept streams."""

    meta: Optional[object] = None
    """
    A user modifiable key-value store used to reference other systems of record for
    managing live inputs.
    """

    modified: Optional[datetime] = None
    """The date and time the live input was last modified."""

    recording: Optional[Recording] = None
    """Records the input to a Cloudflare Stream video.

    Behavior depends on the mode. In most cases, the video will initially be
    viewable as a live video and transition to on-demand after a condition is
    satisfied.
    """

    rtmps: Optional[Rtmps] = None
    """Details for streaming to an live input using RTMPS."""

    rtmps_playback: Optional[RtmpsPlayback] = FieldInfo(alias="rtmpsPlayback", default=None)
    """Details for playback from an live input using RTMPS."""

    srt: Optional[Srt] = None
    """Details for streaming to a live input using SRT."""

    srt_playback: Optional[SrtPlayback] = FieldInfo(alias="srtPlayback", default=None)
    """Details for playback from an live input using SRT."""

    status: Optional[
        Literal[
            "connected",
            "reconnected",
            "reconnecting",
            "client_disconnect",
            "ttl_exceeded",
            "failed_to_connect",
            "failed_to_reconnect",
            "new_configuration_accepted",
        ]
    ] = None
    """The connection status of a live input."""

    uid: Optional[str] = None
    """A unique identifier for a live input."""

    web_rtc: Optional[WebRtc] = FieldInfo(alias="webRTC", default=None)
    """Details for streaming to a live input using WebRTC."""

    web_rtc_playback: Optional[WebRtcPlayback] = FieldInfo(alias="webRTCPlayback", default=None)
    """Details for playback from a live input using WebRTC."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .hijacks import (
    HijacksResource,
    AsyncHijacksResource,
    HijacksResourceWithRawResponse,
    AsyncHijacksResourceWithRawResponse,
    HijacksResourceWithStreamingResponse,
    AsyncHijacksResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "HijacksResource",
    "AsyncHijacksResource",
    "HijacksResourceWithRawResponse",
    "AsyncHijacksResourceWithRawResponse",
    "HijacksResourceWithStreamingResponse",
    "AsyncHijacksResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, cast
from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.networks.route import Route
from .....types.zero_trust.networks.routes import network_create_params, network_delete_params

__all__ = ["NetworksResource", "AsyncNetworksResource"]


class NetworksResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> NetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return NetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> NetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return NetworksResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "This endpoint and its related APIs are deprecated in favor of the equivalent Tunnel Route (without CIDR) APIs."
    )
    def create(
        self,
        ip_network_encoded: str,
        *,
        account_id: str,
        tunnel_id: str,
        comment: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Routes a private network through a Cloudflare Tunnel.

        The CIDR in
        `ip_network_encoded` must be written in URL-encoded format.

        Args:
          account_id: Cloudflare account ID

          ip_network_encoded: IP/CIDR range in URL-encoded format

          tunnel_id: UUID of the tunnel.

          comment: Optional remark describing the route.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ip_network_encoded:
            raise ValueError(f"Expected a non-empty value for `ip_network_encoded` but received {ip_network_encoded!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/teamnet/routes/network/{ip_network_encoded}",
                account_id=account_id,
                ip_network_encoded=ip_network_encoded,
            ),
            body=maybe_transform(
                {
                    "tunnel_id": tunnel_id,
                    "comment": comment,
                    "virtual_network_id": virtual_network_id,
                },
                network_create_params.NetworkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    @typing_extensions.deprecated(
        "This endpoint and its related APIs are deprecated in favor of the equivalent Tunnel Route (without CIDR) APIs."
    )
    def delete(
        self,
        ip_network_encoded: str,
        *,
        account_id: str,
        tun_type: Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"] | Omit = omit,
        tunnel_id: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Deletes a private network route from an account.

        The CIDR in
        `ip_network_encoded` must be written in URL-encoded format. If no
        virtual_network_id is provided it will delete the route from the default vnet.
        If no tun_type is provided it will fetch the type from the tunnel_id or if that
        is missing it will assume Cloudflare Tunnel as default. If tunnel_id is provided
        it will delete the route from that tunnel, otherwise it will delete the route
        based on the vnet and tun_type.

        Args:
          account_id: Cloudflare account ID

          ip_network_encoded: IP/CIDR range in URL-encoded format

          tun_type: The type of tunnel.

          tunnel_id: UUID of the tunnel.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ip_network_encoded:
            raise ValueError(f"Expected a non-empty value for `ip_network_encoded` but received {ip_network_encoded!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/teamnet/routes/network/{ip_network_encoded}",
                account_id=account_id,
                ip_network_encoded=ip_network_encoded,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "tun_type": tun_type,
                        "tunnel_id": tunnel_id,
                        "virtual_network_id": virtual_network_id,
                    },
                    network_delete_params.NetworkDeleteParams,
                ),
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    @typing_extensions.deprecated(
        "This endpoint and its related APIs are deprecated in favor of the equivalent Tunnel Route (without CIDR) APIs."
    )
    def edit(
        self,
        ip_network_encoded: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Updates an existing private network route in an account.

        The CIDR in
        `ip_network_encoded` must be written in URL-encoded format.

        Args:
          account_id: Cloudflare account ID

          ip_network_encoded: IP/CIDR range in URL-encoded format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ip_network_encoded:
            raise ValueError(f"Expected a non-empty value for `ip_network_encoded` but received {ip_network_encoded!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/teamnet/routes/network/{ip_network_encoded}",
                account_id=account_id,
                ip_network_encoded=ip_network_encoded,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )


class AsyncNetworksResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncNetworksResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncNetworksResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncNetworksResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncNetworksResourceWithStreamingResponse(self)

    @typing_extensions.deprecated(
        "This endpoint and its related APIs are deprecated in favor of the equivalent Tunnel Route (without CIDR) APIs."
    )
    async def create(
        self,
        ip_network_encoded: str,
        *,
        account_id: str,
        tunnel_id: str,
        comment: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Routes a private network through a Cloudflare Tunnel.

        The CIDR in
        `ip_network_encoded` must be written in URL-encoded format.

        Args:
          account_id: Cloudflare account ID

          ip_network_encoded: IP/CIDR range in URL-encoded format

          tunnel_id: UUID of the tunnel.

          comment: Optional remark describing the route.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ip_network_encoded:
            raise ValueError(f"Expected a non-empty value for `ip_network_encoded` but received {ip_network_encoded!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/teamnet/routes/network/{ip_network_encoded}",
                account_id=account_id,
                ip_network_encoded=ip_network_encoded,
            ),
            body=await async_maybe_transform(
                {
                    "tunnel_id": tunnel_id,
                    "comment": comment,
                    "virtual_network_id": virtual_network_id,
                },
                network_create_params.NetworkCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    @typing_extensions.deprecated(
        "This endpoint and its related APIs are deprecated in favor of the equivalent Tunnel Route (without CIDR) APIs."
    )
    async def delete(
        self,
        ip_network_encoded: str,
        *,
        account_id: str,
        tun_type: Literal["cfd_tunnel", "warp_connector", "warp", "magic", "ip_sec", "gre", "cni"] | Omit = omit,
        tunnel_id: str | Omit = omit,
        virtual_network_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Deletes a private network route from an account.

        The CIDR in
        `ip_network_encoded` must be written in URL-encoded format. If no
        virtual_network_id is provided it will delete the route from the default vnet.
        If no tun_type is provided it will fetch the type from the tunnel_id or if that
        is missing it will assume Cloudflare Tunnel as default. If tunnel_id is provided
        it will delete the route from that tunnel, otherwise it will delete the route
        based on the vnet and tun_type.

        Args:
          account_id: Cloudflare account ID

          ip_network_encoded: IP/CIDR range in URL-encoded format

          tun_type: The type of tunnel.

          tunnel_id: UUID of the tunnel.

          virtual_network_id: UUID of the virtual network.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ip_network_encoded:
            raise ValueError(f"Expected a non-empty value for `ip_network_encoded` but received {ip_network_encoded!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/teamnet/routes/network/{ip_network_encoded}",
                account_id=account_id,
                ip_network_encoded=ip_network_encoded,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "tun_type": tun_type,
                        "tunnel_id": tunnel_id,
                        "virtual_network_id": virtual_network_id,
                    },
                    network_delete_params.NetworkDeleteParams,
                ),
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )

    @typing_extensions.deprecated(
        "This endpoint and its related APIs are deprecated in favor of the equivalent Tunnel Route (without CIDR) APIs."
    )
    async def edit(
        self,
        ip_network_encoded: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Route:
        """Updates an existing private network route in an account.

        The CIDR in
        `ip_network_encoded` must be written in URL-encoded format.

        Args:
          account_id: Cloudflare account ID

          ip_network_encoded: IP/CIDR range in URL-encoded format

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not ip_network_encoded:
            raise ValueError(f"Expected a non-empty value for `ip_network_encoded` but received {ip_network_encoded!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/teamnet/routes/network/{ip_network_encoded}",
                account_id=account_id,
                ip_network_encoded=ip_network_encoded,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Route]._unwrapper,
            ),
            cast_to=cast(Type[Route], ResultWrapper[Route]),
        )


class NetworksResourceWithRawResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

        self.create = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                networks.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                networks.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                networks.edit,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncNetworksResourceWithRawResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                networks.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                networks.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                networks.edit,  # pyright: ignore[reportDeprecated],
            )
        )


class NetworksResourceWithStreamingResponse:
    def __init__(self, networks: NetworksResource) -> None:
        self._networks = networks

        self.create = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                networks.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                networks.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                networks.edit,  # pyright: ignore[reportDeprecated],
            )
        )


class AsyncNetworksResourceWithStreamingResponse:
    def __init__(self, networks: AsyncNetworksResource) -> None:
        self._networks = networks

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                networks.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                networks.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.edit = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                networks.edit,  # pyright: ignore[reportDeprecated],
            )
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .payloads import (
    PayloadsResource,
    AsyncPayloadsResource,
    PayloadsResourceWithRawResponse,
    AsyncPayloadsResourceWithRawResponse,
    PayloadsResourceWithStreamingResponse,
    AsyncPayloadsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from .content_scanning import (
    ContentScanningResource,
    AsyncContentScanningResource,
    ContentScanningResourceWithRawResponse,
    AsyncContentScanningResourceWithRawResponse,
    ContentScanningResourceWithStreamingResponse,
    AsyncContentScanningResourceWithStreamingResponse,
)

__all__ = [
    "PayloadsResource",
    "AsyncPayloadsResource",
    "PayloadsResourceWithRawResponse",
    "AsyncPayloadsResourceWithRawResponse",
    "PayloadsResourceWithStreamingResponse",
    "AsyncPayloadsResourceWithStreamingResponse",
    "SettingsResource",
    "AsyncSettingsResource",
    "SettingsResourceWithRawResponse",
    "AsyncSettingsResourceWithRawResponse",
    "SettingsResourceWithStreamingResponse",
    "AsyncSettingsResourceWithStreamingResponse",
    "ContentScanningResource",
    "AsyncContentScanningResource",
    "ContentScanningResourceWithRawResponse",
    "AsyncContentScanningResourceWithRawResponse",
    "ContentScanningResourceWithStreamingResponse",
    "AsyncContentScanningResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import TypeAlias

from ..._models import BaseModel

__all__ = ["IPListResponse", "PublicIPIPs", "PublicIPIPsJDCloud"]


class PublicIPIPs(BaseModel):
    etag: Optional[str] = None
    """A digest of the IP data. Useful for determining if the data has changed."""

    ipv4_cidrs: Optional[List[str]] = None
    """List of Cloudflare IPv4 CIDR addresses."""

    ipv6_cidrs: Optional[List[str]] = None
    """List of Cloudflare IPv6 CIDR addresses."""


class PublicIPIPsJDCloud(BaseModel):
    etag: Optional[str] = None
    """A digest of the IP data. Useful for determining if the data has changed."""

    ipv4_cidrs: Optional[List[str]] = None
    """List of Cloudflare IPv4 CIDR addresses."""

    ipv6_cidrs: Optional[List[str]] = None
    """List of Cloudflare IPv6 CIDR addresses."""

    jdcloud_cidrs: Optional[List[str]] = None
    """List IPv4 and IPv6 CIDRs, only populated if `?networks=jdcloud` is used."""


IPListResponse: TypeAlias = Union[PublicIPIPs, PublicIPIPsJDCloud]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["SilenceUpdateResponse"]


class SilenceUpdateResponse(BaseModel):
    id: Optional[str] = None
    """Silence ID"""

    created_at: Optional[str] = None
    """When the silence was created."""

    end_time: Optional[str] = None
    """When the silence ends."""

    policy_id: Optional[str] = None
    """The unique identifier of a notification policy"""

    start_time: Optional[str] = None
    """When the silence starts."""

    updated_at: Optional[str] = None
    """When the silence was modified."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._types import SequenceNotStr
from ...._utils import PropertyInfo

__all__ = ["AttackerListParams"]


class AttackerListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account ID."""

    dataset_ids: Annotated[SequenceNotStr[str], PropertyInfo(alias="datasetIds")]
    """Array of dataset IDs to query attackers from.

    If not provided, uses the default dataset.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = ["EligibleGetResponse", "EligibleGetResponseItem"]


class EligibleGetResponseItem(BaseModel):
    eligible: Optional[bool] = None
    """Determines whether or not the account is eligible for the delivery mechanism."""

    ready: Optional[bool] = None
    """Beta flag.

    Users can create a policy with a mechanism that is not ready, but we cannot
    guarantee successful delivery of notifications.
    """

    type: Optional[Literal["email", "pagerduty", "webhook"]] = None
    """Determines type of delivery mechanism."""


EligibleGetResponse: TypeAlias = Dict[str, List[EligibleGetResponseItem]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ContentList"]


class ContentList(BaseModel):
    action: Optional[Literal["block"]] = None
    """Behavior of the content list."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .bgp_prefix import BGPPrefix as BGPPrefix
from .delegations import Delegations as Delegations
from .service_binding import ServiceBinding as ServiceBinding
from .bgp_prefix_edit_params import BGPPrefixEditParams as BGPPrefixEditParams
from .bgp_prefix_create_params import BGPPrefixCreateParams as BGPPrefixCreateParams
from .delegation_create_params import DelegationCreateParams as DelegationCreateParams
from .delegation_delete_response import DelegationDeleteResponse as DelegationDeleteResponse
from .service_binding_create_params import ServiceBindingCreateParams as ServiceBindingCreateParams
from .service_binding_delete_response import ServiceBindingDeleteResponse as ServiceBindingDeleteResponse
from .advertisement_status_edit_params import AdvertisementStatusEditParams as AdvertisementStatusEditParams
from .advertisement_status_get_response import AdvertisementStatusGetResponse as AdvertisementStatusGetResponse
from .advertisement_status_edit_response import AdvertisementStatusEditResponse as AdvertisementStatusEditResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .custom import (
    CustomResource,
    AsyncCustomResource,
    CustomResourceWithRawResponse,
    AsyncCustomResourceWithRawResponse,
    CustomResourceWithStreamingResponse,
    AsyncCustomResourceWithStreamingResponse,
)
from .managed import (
    ManagedResource,
    AsyncManagedResource,
    ManagedResourceWithRawResponse,
    AsyncManagedResourceWithRawResponse,
    ManagedResourceWithStreamingResponse,
    AsyncManagedResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource

__all__ = ["DomainsResource", "AsyncDomainsResource"]


class DomainsResource(SyncAPIResource):
    @cached_property
    def custom(self) -> CustomResource:
        return CustomResource(self._client)

    @cached_property
    def managed(self) -> ManagedResource:
        return ManagedResource(self._client)

    @cached_property
    def with_raw_response(self) -> DomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DomainsResourceWithStreamingResponse(self)


class AsyncDomainsResource(AsyncAPIResource):
    @cached_property
    def custom(self) -> AsyncCustomResource:
        return AsyncCustomResource(self._client)

    @cached_property
    def managed(self) -> AsyncManagedResource:
        return AsyncManagedResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDomainsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDomainsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDomainsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDomainsResourceWithStreamingResponse(self)


class DomainsResourceWithRawResponse:
    def __init__(self, domains: DomainsResource) -> None:
        self._domains = domains

    @cached_property
    def custom(self) -> CustomResourceWithRawResponse:
        return CustomResourceWithRawResponse(self._domains.custom)

    @cached_property
    def managed(self) -> ManagedResourceWithRawResponse:
        return ManagedResourceWithRawResponse(self._domains.managed)


class AsyncDomainsResourceWithRawResponse:
    def __init__(self, domains: AsyncDomainsResource) -> None:
        self._domains = domains

    @cached_property
    def custom(self) -> AsyncCustomResourceWithRawResponse:
        return AsyncCustomResourceWithRawResponse(self._domains.custom)

    @cached_property
    def managed(self) -> AsyncManagedResourceWithRawResponse:
        return AsyncManagedResourceWithRawResponse(self._domains.managed)


class DomainsResourceWithStreamingResponse:
    def __init__(self, domains: DomainsResource) -> None:
        self._domains = domains

    @cached_property
    def custom(self) -> CustomResourceWithStreamingResponse:
        return CustomResourceWithStreamingResponse(self._domains.custom)

    @cached_property
    def managed(self) -> ManagedResourceWithStreamingResponse:
        return ManagedResourceWithStreamingResponse(self._domains.managed)


class AsyncDomainsResourceWithStreamingResponse:
    def __init__(self, domains: AsyncDomainsResource) -> None:
        self._domains = domains

    @cached_property
    def custom(self) -> AsyncCustomResourceWithStreamingResponse:
        return AsyncCustomResourceWithStreamingResponse(self._domains.custom)

    @cached_property
    def managed(self) -> AsyncManagedResourceWithStreamingResponse:
        return AsyncManagedResourceWithStreamingResponse(self._domains.managed)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["AppUpdateParams"]


class AppUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    hostnames: SequenceNotStr[str]
    """FQDNs to associate with traffic decisions."""

    ip_subnets: SequenceNotStr[str]
    """IPv4 CIDRs to associate with traffic decisions.

    (IPv6 CIDRs are currently unsupported)
    """

    name: str
    """Display name for the app."""

    source_subnets: SequenceNotStr[str]
    """IPv4 CIDRs to associate with traffic decisions.

    (IPv6 CIDRs are currently unsupported)
    """

    type: str
    """Category of the app."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .logs import (
    LogsResource,
    AsyncLogsResource,
    LogsResourceWithRawResponse,
    AsyncLogsResourceWithRawResponse,
    LogsResourceWithStreamingResponse,
    AsyncLogsResourceWithStreamingResponse,
)

__all__ = [
    "LogsResource",
    "AsyncLogsResource",
    "LogsResourceWithRawResponse",
    "AsyncLogsResourceWithRawResponse",
    "LogsResourceWithStreamingResponse",
    "AsyncLogsResourceWithStreamingResponse",
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
]
